@@ -8,15 +8,11 @@ async fn init_api() -> Rest {
 
     let subaccount = var("SUBACCOUNT").ok();
 
-    let api = Rest::new(
-        var("API_KEY").expect("API Key is not defined."),
-        var("API_SECRET").expect("API Secret is not defined."),
-        var("SUBACCOUNT").ok(),
-    );
+    let api = Rest::new(Options::from_env());
 
     // Test create subaccount only if credentials are account-wide
     if subaccount.is_none() {
-        read_only(api.create_subaccount("Bot").await);
+        read_only(api.request(CreateSubAccount::new("Bot")).await);
     }
 
     api
@@ -34,7 +30,7 @@ async fn get_subaccounts() {
     let rest = init_api().await;
     if rest.subaccount.is_none() {
         // Test only if credentials are account-wide
-        rest.get_subaccounts().await.unwrap();
+        rest.request(GetSubAccounts).await.unwrap();
     }
 }
 
@@ -43,7 +39,7 @@ async fn create_subaccount() {
     let rest = init_api().await;
     if rest.subaccount.is_none() {
         // Test only if credentials are account-wide
-        read_only(rest.create_subaccount("Bot").await);
+        read_only(rest.request(CreateSubAccount::new("Bot")).await);
     }
 }
 
@@ -52,7 +48,7 @@ async fn change_subaccount_name() {
     let rest = init_api().await;
     if rest.subaccount.is_none() {
         // Test only if credentials are account-wide
-        read_only(rest.change_subaccount_name("Bot", "Bot").await);
+        read_only(rest.request(ChangeSubaccountName::new("Bot", "Bot")).await);
     }
 }
 
@@ -61,7 +57,7 @@ async fn delete_subaccount() {
     let rest = init_api().await;
     if rest.subaccount.is_none() {
         // Test only if credentials are account-wide
-        read_only(rest.delete_subaccount("Bot").await);
+        read_only(rest.request(DeleteSubaccount::new("Bot")).await);
     }
 }
 
@@ -73,7 +69,9 @@ async fn get_subaccount_balances() {
         None => "Bot",
         Some(sub) => sub,
     };
-    rest.get_subaccount_balances(&subaccount).await.unwrap_err();
+    rest.request(GetSubaccountBalances::new(subaccount))
+        .await
+        .unwrap_err();
 }
 
 #[tokio::test]
@@ -81,32 +79,41 @@ async fn transfer_between_subaccounts() {
     let rest = init_api().await;
     if rest.subaccount.is_none() {
         // Test only if credentials are account-wide
-        rest.transfer_between_subaccounts("BTC", Decimal::zero(), "Source", "Destination")
-            .await
-            .unwrap_err();
+        rest.request(TransferBetweenSubaccounts::new(
+            "BTC",
+            Decimal::zero(),
+            "Source",
+            "Destination",
+        ))
+        .await
+        .unwrap_err();
     }
 }
 
 #[tokio::test]
 async fn get_markets() {
-    init_api().await.get_markets().await.unwrap();
+    init_api().await.request(GetMarkets).await.unwrap();
 }
 
 #[tokio::test]
 async fn get_market() {
-    init_api().await.get_market("BTC/USD").await.unwrap();
+    init_api()
+        .await
+        .request(GetMarket::new("BTC/USD"))
+        .await
+        .unwrap();
 }
 
 #[tokio::test]
 async fn get_orderbook() {
     init_api()
         .await
-        .get_orderbook("BTC/USD", None)
+        .request(GetOrderBook::new("BTC/USD"))
         .await
         .unwrap();
     init_api()
         .await
-        .get_orderbook("BTC/USD", Some(50))
+        .request(GetOrderBook::with_depth("BTC/USD", 50))
         .await
         .unwrap();
 }
@@ -115,7 +122,7 @@ async fn get_orderbook() {
 async fn get_trades() {
     init_api()
         .await
-        .get_trades("BTC/USD", None, None, None)
+        .request(GetTrades::new("BTC/USD"))
         .await
         .unwrap();
 }
@@ -124,19 +131,27 @@ async fn get_trades() {
 async fn get_historical_prices() {
     init_api()
         .await
-        .get_historical_prices("BTC/USD", 300, None, None, None)
+        .request(GetHistoricalPrices {
+            market_name: "BTC/USD".into(),
+            resolution: 300,
+            ..Default::default()
+        })
         .await
         .unwrap();
 }
 
 #[tokio::test]
 async fn get_futures() {
-    init_api().await.get_futures().await.unwrap();
+    init_api().await.request(GetFutures).await.unwrap();
 }
 
 #[tokio::test]
 async fn get_future() {
-    init_api().await.get_future("BTC-PERP").await.unwrap();
+    init_api()
+        .await
+        .request(GetFuture::new("BTC-PERP"))
+        .await
+        .unwrap();
 }
 
 #[tokio::test]
@@ -185,7 +200,17 @@ async fn account_deserialization() {
 }
 
 #[tokio::test]
-async fn place_modify_cancel_order() {
+async fn get_coins() {
+    init_api().await.request(GetCoins).await.unwrap();
+}
+
+#[tokio::test]
+async fn place_modify_cancel_orders() {
+    manipulate_orders().await;
+}
+
+// Helper function used in place_modify_cancel_orders and ws::tests::orders
+pub async fn manipulate_orders() {
     let api = init_api().await;
     // Testing with ETH since BTC's minimum provide (maker) size is 0.01 BTC,
     // too large for testing purposes
@@ -210,17 +235,15 @@ async fn place_modify_cancel_order() {
 
     // Test place order
     let initial_order = api
-        .place_order(
-            market.as_str(),
-            OrderSide::Buy,
-            Some(initial_bid_price),
-            OrderType::Limit,
-            initial_bid_size,
-            None,
-            None,
-            Some(true),
-            None,
-        )
+        .request(PlaceOrder {
+            market: market.clone(),
+            side: Side::Buy,
+            price: Some(initial_bid_price),
+            r#type: OrderType::Limit,
+            size: initial_bid_size,
+            post_only: true,
+            ..Default::default()
+        })
         .await
         .unwrap();
     // println!("Initial order: {:?}", initial_order);
@@ -229,12 +252,12 @@ async fn place_modify_cancel_order() {
 
     // Test modify order
     let modified_order = api
-        .modify_order(
-            initial_order.id,
-            Some(modified_bid_price),
-            Some(modified_bid_size),
-            None,
-        )
+        .request(ModifyOrder {
+            id: initial_order.id,
+            price: Some(modified_bid_price),
+            size: Some(modified_bid_size),
+            ..Default::default()
+        })
         .await
         .unwrap();
     // println!("Modified order: {:?}", modified_order);
@@ -245,7 +268,10 @@ async fn place_modify_cancel_order() {
     assert_eq!(modified_bid_size, modified_order.size);
 
     // Test cancel order
-    let cancelled_response = api.cancel_order(modified_order.id).await.unwrap();
+    let cancelled_response = api
+        .request(CancelOrder::new(modified_order.id))
+        .await
+        .unwrap();
     // println!("Cancelled response: {:?}", cancelled_response);
     assert_eq!(
         "Order queued for cancellation".to_string(),
@@ -253,10 +279,29 @@ async fn place_modify_cancel_order() {
     );
 
     // Check that order was actually cancelled
-    let cancelled_order = api.get_order(modified_order.id).await.unwrap();
+    let cancelled_order = api.request(GetOrder::new(modified_order.id)).await.unwrap();
     // println!("Cancelled order: {:?}", cancelled_order);
     assert_eq!(modified_order.id, cancelled_order.id);
-    assert_eq!(dec!(0), cancelled_order.filled_size);
+    assert_eq!(dec!(0), cancelled_order.filled_size.unwrap());
     assert_eq!(None, cancelled_order.avg_fill_price);
     assert_eq!(OrderStatus::Closed, cancelled_order.status);
+
+    // Place a post-only order that will be rejected
+    let rejected_bid_price = dec!(1.1) * price; // Bid at 110% of current price
+    let rejected_order = api
+        .request(PlaceOrder {
+            market,
+            side: Side::Buy,
+            price: Some(rejected_bid_price),
+            r#type: OrderType::Limit,
+            size: initial_bid_size,
+            post_only: true,
+            ..Default::default()
+        })
+        .await
+        .unwrap();
+    assert_eq!(dec!(0), rejected_order.filled_size.unwrap());
+    assert_eq!(None, rejected_order.avg_fill_price);
+
+    assert_eq!(OrderStatus::New, rejected_order.status);
 }
