@@ -8,10 +8,16 @@ mod tests;
 pub use error::*;
 pub use model::*;
 
-use futures_util::{SinkExt, StreamExt};
+use crate::options::Options;
+use futures::{
+    ready,
+    task::{Context, Poll},
+    Future, SinkExt, Stream, StreamExt,
+};
 use hmac_sha256::HMAC;
 use serde_json::json;
 use std::collections::VecDeque;
+use std::pin::Pin;
 use std::time::{Duration, SystemTime, UNIX_EPOCH};
 use tokio::net::TcpStream;
 use tokio::time; // 1.3.0
@@ -21,65 +27,54 @@ use tokio_tungstenite::{connect_async, tungstenite::Message, MaybeTlsStream, Web
 pub struct Ws {
     channels: Vec<Channel>,
     stream: WebSocketStream<MaybeTlsStream<TcpStream>>,
-    buf: VecDeque<Data>,
+    buf: VecDeque<(Option<Symbol>, Data)>,
     ping_timer: Interval,
+    /// Whether the websocket was opened authenticated with API keys or not
+    is_authenticated: bool,
 }
 
 impl Ws {
     pub const ENDPOINT: &'static str = "wss://ftx.com/ws";
     pub const ENDPOINT_US: &'static str = "wss://ftx.us/ws";
 
-    async fn connect_with_endpoint(
-        endpoint: &str,
-        key: String,
-        secret: String,
-        subaccount: Option<String>,
-    ) -> Result<Self> {
-        let (mut stream, _) = connect_async(endpoint).await?;
-
-        let timestamp = SystemTime::now()
-            .duration_since(UNIX_EPOCH)
-            .unwrap()
-            .as_millis();
-        let sign_payload = format!("{}websocket_login", timestamp);
-        let sign = HMAC::mac(sign_payload.as_bytes(), secret.as_bytes());
-        let sign = hex::encode(sign);
-
-        stream
-            .send(Message::Text(
-                json!({
-                    "op": "login",
-                    "args": {
-                        "key": key,
-                        "sign": sign,
-                        "time": timestamp as u64,
-                        "subaccount": subaccount,
-                    }
-                })
-                .to_string(),
-            ))
-            .await?;
+    pub async fn connect(options: Options) -> Result<Self> {
+        let (mut stream, _) = connect_async(options.endpoint.ws()).await?;
+        let is_authenticated = if let (Some(key), Some(secret)) = (options.key, options.secret) {
+            let timestamp = SystemTime::now()
+                .duration_since(UNIX_EPOCH)
+                .unwrap()
+                .as_millis();
+            let sign_payload = format!("{}websocket_login", timestamp);
+            let sign = HMAC::mac(sign_payload.as_bytes(), secret.as_bytes());
+            let sign = hex::encode(sign);
 
+            stream
+                .send(Message::Text(
+                    json!({
+                        "op": "login",
+                        "args": {
+                            "key": key,
+                            "sign": sign,
+                            "time": timestamp as u64,
+                            "subaccount": options.subaccount,
+                        }
+                    })
+                    .to_string(),
+                ))
+                .await?;
+            true
+        } else {
+            false
+        };
         Ok(Self {
             channels: Vec::new(),
             stream,
             buf: VecDeque::new(),
             ping_timer: time::interval(Duration::from_secs(15)),
+            is_authenticated,
         })
     }
 
-    pub async fn connect(key: String, secret: String, subaccount: Option<String>) -> Result<Self> {
-        Self::connect_with_endpoint(Self::ENDPOINT, key, secret, subaccount).await
-    }
-
-    pub async fn connect_us(
-        key: String,
-        secret: String,
-        subaccount: Option<String>,
-    ) -> Result<Self> {
-        Self::connect_with_endpoint(Self::ENDPOINT_US, key, secret, subaccount).await
-    }
-
     async fn ping(&mut self) -> Result<()> {
         self.stream
             .send(Message::Text(
@@ -94,8 +89,14 @@ impl Ws {
     }
 
     /// Subscribe to specified `Channel`s
+    /// For FILLS the socket needs to be authenticated
     pub async fn subscribe(&mut self, channels: Vec<Channel>) -> Result<()> {
         for channel in channels.iter() {
+            // Subscribing to fills or orders requires us to be authenticated via an API key
+            if (channel == &Channel::Fills || channel == &Channel::Orders) && !self.is_authenticated
+            {
+                return Err(Error::SocketNotAuthenticated);
+            }
             self.channels.push(channel.clone());
         }
 
@@ -108,7 +109,7 @@ impl Ws {
     pub async fn unsubscribe(&mut self, channels: Vec<Channel>) -> Result<()> {
         // Check that the specified channels match an existing one
         for channel in channels.iter() {
-            if !self.channels.contains(&channel) {
+            if !self.channels.contains(channel) {
                 return Err(Error::NotSubscribedToThisChannel(channel.clone()));
             }
         }
@@ -148,6 +149,7 @@ impl Ws {
                 Channel::Trades(symbol) => ("trades", symbol),
                 Channel::Ticker(symbol) => ("ticker", symbol),
                 Channel::Fills => ("fills", "".to_string()),
+                Channel::Orders => ("orders", "".to_string()),
             };
 
             self.stream
@@ -224,31 +226,55 @@ impl Ws {
                     // Trades channel returns an array of single trades.
                     // Buffer so that the user receives trades one at a time
                     for trade in trades {
-                        self.buf.push_back(Data::Trade(trade));
+                        self.buf
+                            .push_back((response.market.clone(), Data::Trade(trade)));
                     }
                 }
                 ResponseData::OrderbookData(orderbook) => {
-                    self.buf.push_back(Data::OrderbookData(orderbook));
+                    self.buf
+                        .push_back((response.market, Data::OrderbookData(orderbook)));
                 }
                 ResponseData::Fill(fill) => {
-                    self.buf.push_back(Data::Fill(fill));
+                    self.buf.push_back((response.market, Data::Fill(fill)));
+                }
+                ResponseData::Ticker(ticker) => {
+                    self.buf.push_back((response.market, Data::Ticker(ticker)));
+                }
+                ResponseData::Order(order) => {
+                    self.buf.push_back((response.market, Data::Order(order)));
                 }
             }
         }
     }
+}
 
-    pub async fn next(&mut self) -> Result<Option<Data>> {
+impl Stream for Ws {
+    type Item = Result<(Option<Symbol>, Data)>;
+
+    fn poll_next(mut self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<Option<Self::Item>> {
         loop {
-            // If buffer contains data, we can directly return it.
             if let Some(data) = self.buf.pop_front() {
-                return Ok(Some(data));
+                return Poll::Ready(Some(Ok(data)));
             }
-
-            // Fetch new response if buffer is empty.
-            let response = self.next_response().await?;
-
+            let response = {
+                // Fetch new response if buffer is empty.
+                // safety: this is ok because the future from self.next_response() will only live in this function.
+                // It won't be moved anymore.
+                let mut next_response = self.next_response();
+                let pinned = unsafe { Pin::new_unchecked(&mut next_response) };
+                match ready!(pinned.poll(cx)) {
+                    Ok(response) => response,
+                    Err(e) => {
+                        return Poll::Ready(Some(Err(e)));
+                    }
+                }
+            };
             // Handle the response, possibly adding to the buffer
             self.handle_response(response);
         }
     }
+
+    fn size_hint(&self) -> (usize, Option<usize>) {
+        (self.buf.len(), None)
+    }
 }
