@@ -1,4 +1,4 @@
-pub use crate::rest::{Coin, Id, MarketType, Side, Symbol};
+pub use crate::rest::{Coin, Id, MarketType, OrderInfo, Side, Symbol};
 use chrono::{DateTime, Utc};
 use crc32fast::Hasher;
 use rust_decimal::Decimal;
@@ -14,6 +14,7 @@ pub enum Channel {
     Trades(Symbol),
     Ticker(Symbol),
     Fills,
+    Orders,
 }
 
 /*
@@ -42,7 +43,7 @@ pub enum Type {
     Error,
     Partial,
     Pong,
-    // Info,         // May need this in the future
+    Info, // May need this in the future
 }
 
 /// Represents the response received from FTX, and is used for
@@ -51,17 +52,34 @@ pub enum Type {
 #[serde(rename_all = "camelCase")]
 #[serde(untagged)]
 pub enum ResponseData {
+    Ticker(Ticker),
     Trades(Vec<Trade>),
     OrderbookData(OrderbookData),
     Fill(Fill),
+    Order(OrderInfo),
 }
 
 /// Represents the data we return to the user
 #[derive(Clone, Debug)]
 pub enum Data {
+    Ticker(Ticker),
     Trade(Trade),
     OrderbookData(OrderbookData),
     Fill(Fill),
+    Order(OrderInfo),
+}
+
+#[serde_as]
+#[derive(Copy, Clone, Debug, Deserialize)]
+#[serde(rename_all = "camelCase")]
+pub struct Ticker {
+    pub bid: Decimal,
+    pub ask: Decimal,
+    pub bid_size: Decimal,
+    pub ask_size: Decimal,
+    pub last: Decimal,
+    #[serde_as(as = "TimestampSecondsWithFrac<f64>")]
+    pub time: DateTime<Utc>,
 }
 
 #[derive(Copy, Clone, Debug, Deserialize)]
@@ -123,6 +141,8 @@ impl Orderbook {
     pub fn update(&mut self, data: &OrderbookData) {
         match data.action {
             OrderbookAction::Partial => {
+                self.bids.clear();
+                self.asks.clear();
                 for bid in &data.bids {
                     self.bids.insert(bid.0, bid.1);
                 }
